clear all; close all;
[filename, pathname] = uigetfile('*.mtx','Choose a matrix');
[dum, name] = fileparts(filename);
matrix = [name,'.mtx'];
rhs  = [name,'_b.mtx'];
[A, n, m, entries, rep, field, symm] = mmread(matrix);
[b, n, m, entries, rep, field, symm] = mmread(rhs);
disp(['The size of the matrix is ',num2str(n)]);
disp(['The matrix is ', symm]);
spy(A);
x0 = zeros(n,1);

choice = menu('Choose a preconditioner: ', 'None', 'Block diagonal','diagonal', 'Block upper triangular','Strictly upper triangular' );
if ( choice == 1 )
   P = [];
elseif ( choice == 2 )
% Find the blocks:
   bs = max(find(diag(A) > 0));
   A11 = A(1:bs,1:bs);
   A21 = A(bs+1:n,1:bs);
   A12 = inv(A11)*A21';
   A22 = A(bs+1:n,bs+1:n);
   [nr nc] = size(A21);
   Z = spalloc(nr,nc,0);
   S = A21*A12-A22;
   P = [A11 Z'; Z S];
elseif ( choice == 3 )
% Find the blocks:
   bs = max(find(diag(A) > 0));
   A11 = A(1:bs,1:bs);
   A21 = A(bs+1:n,1:bs);
   A12 = inv(A11)*A21';
   A22 = A(bs+1:n,bs+1:n);
   [nr nc] = size(A21);
   Z = spalloc(nr,nc,0);
   S = -A22;
   for i = 1:nr
      S(i,i) = S(i,i) + dot(A21(i,:),A12(:,i));
   end
   P = [A11 Z'; Z S];
elseif ( choice == 4 )
% Find the blocks:
   bs = max(find(diag(A) > 0));
   A11 = A(1:bs,1:bs);
   A21 = A(bs+1:n,1:bs);
   A12 = inv(A11)*A21';
   A22 = A(bs+1:n,bs+1:n);
   [nr nc] = size(A21);
   Z = spalloc(nr,nc,0);
   S = A21*A12-A22;
   P = [A11 A21'; Z -S];
else
% Find the blocks:
   bs = max(find(diag(A) > 0));
   A11 = A(1:bs,1:bs);
   A21 = A(bs+1:n,1:bs);
   A12 = inv(A11)*A21';
   A22 = A(bs+1:n,bs+1:n);
   [nr nc] = size(A21);
   Z = spalloc(nr,nc,0);
   S = -A22;
   for i = 1:nr
      S(i,i) = S(i,i) + dot(A21(i,:),A12(:,i));
   end
   P = [A11 A21'; Z -S];
end

tol = 1e-8;
max_it = 2000;

choice = 1;
fig = figure;
hold on;
xlabel('Number of MATVECS')
ylabel('|r|/|b|')
title(['Convergence for ',filename] );
methods = [];
c = ['b','g','r','c','m','y','k'];
k = 0;
method = char('MINRES', 'GMRES','Bi-CG','QMR','Bi-CGSTAB','IDR(1)','IDR(4)');
methods = [];
while ( choice < 8 )
   choice = menu('Choose a solver: ', 'MINRES', 'GMRES','Bi-CG','QMR','Bi-CGSTAB','IDR(1)','IDR(4)','STOP' );
   k = k + 1;
   if k > 7
      k = 1
   end
   if ( choice == 1 )
      disp('MINRES ...')
      methods = [methods;method(choice,:)]; 
      tic
      [x,flag,relres,iter,resvec] = minres(A,b,tol,max_it,P);
      toc
      true_err = norm(b-A*x)/norm(b);
      disp(['|b-Ax|/|b| = ',num2str(true_err)]);
      disp(['MINRES iterations: ',num2str(iter)])
   elseif ( choice == 2 )
      disp('GMRES ...')
      methods = [methods;method(choice,:)];
      tic
      [x,flag,relres,iter,resvec] = gmres(A,b,[],tol,max_it,P);
      toc
      true_err = norm(b-A*x)/norm(b);
      disp(['|b-Ax|/|b| = ',num2str(true_err)]);
      disp(['GMRES iterations: ',num2str(iter(2))])
   elseif ( choice == 3 )
      disp('Bi-CG ...')
      methods = [methods;method(choice,:)];
      tic
      [x,flag,relres,iter,resvec] = bicg(A,b,tol,max_it,P);
      toc
      true_err = norm(b-A*x)/norm(b);
      disp(['|b-Ax|/|b| = ',num2str(true_err)]);
      disp(['Bi-CG iterations: ',num2str(iter)])
   elseif ( choice == 4 )
      disp('QMR ...')
      methods = [methods;method(choice,:)];
      tic
      [x,flag,relres,iter,resvec] = qmr(A,b,tol,max_it,P);
      toc
      true_err = norm(b-A*x)/norm(b);
      disp(['|b-Ax|/|b| = ',num2str(true_err)]);
      disp(['QMR iterations: ',num2str(iter)])
   elseif ( choice == 5 )
      disp('Bi-CGSTAB ...')
      methods = [methods;method(choice,:)];
      tic
      [x,flag,relres,iter,resvec] = bicgstab(A,b,tol,max_it,P);
      toc
      true_err = norm(b-A*x)/norm(b);
      disp(['|b-Ax|/|b| = ',num2str(true_err)]);
      disp(['Bi-CGSTAB iterations: ',num2str(iter)])
   elseif ( choice == 6 )
      disp('IDR(1) ...')
      methods = [methods;method(choice,:)];
      tic
      [x,flag,relres,iter,resvec] = idrs(A,b,1,tol,max_it,P);
      toc
      true_err = norm(b-A*x)/norm(b);
      disp(['|b-Ax|/|b| = ',num2str(true_err)]);
      disp(['IDR(1) iterations: ',num2str(iter)])
   elseif ( choice == 7 )
      disp('IDR(4) ...')
      methods = [methods;method(choice,:)];
      tic
      [x,flag,relres,iter,resvec] = idrs(A,b,4,tol,max_it,P);
      toc
      true_err = norm(b-A*x)/norm(b);
      disp(['|b-Ax|/|b| = ',num2str(true_err)]);
      disp(['IDR(4) iterations: ',num2str(iter)])
   elseif ( choice == 8 )
      close all;
      break
   end
   figure(fig);
   if ( choice == 3 | choice == 4 )
      x = 2*[1:1:length(resvec)]-1;
   else
      x = [1:1:length(resvec)];
   end 
   resvec = log10(resvec/norm(b));
   plot(x,resvec,c(k));
   legend(methods);
end
return
